<?php

namespace Teknisa\EST\Controllers;

use \Zeedhi\Framework\DTO;
use \Zeedhi\Framework\DTO\Response\Error;
use \Zeedhi\Framework\DataSource\Manager;
use \Zeedhi\Framework\DataSource\DataSet;
use \Zeedhi\Framework\DataSource\FilterCriteria;
use \Zeedhi\Framework\DataSource\ParameterBag;
use \Zeedhi\Framework\DTO\Response\Message;
use Doctrine\DBAL\Connection;

class ProcProdWorkflow extends \Teknisa\EST\Controllers\ProdutoCrud{

	protected $dataSourceName;
	protected $dataSourceManager;
	protected $crudService;
	protected $environment;
	protected $parameterBagColumns = array();
	protected $newCode;
    protected $entityManager;
	const PRODUTO_ATIVO = "S";
	const PRODUTO_INATIVO = "N";

	public function __construct($crudService, ParameterBag $parameterBag, $newCode,  $environment, $dataSourceManager, $entityManager){
		$this->crudService = $crudService;
		$this->dataSourceManager = $dataSourceManager;
		$this->newCode = $newCode;
		$this->environment = $environment;
		parent::__construct($crudService, $parameterBag, $newCode, $environment);
		$this->entityManager = $entityManager;
	}

	public function procProdWorkflow(DTO\Request $request, DTO\Response $response){
		$connection = $this->entityManager->getConnection();
		try {
			$connection->beginTransaction();
			$row = $request->getRow();
			if($row['IDPRODATIVPR']){
				if($row['IDPRODATIVPR'] === self::PRODUTO_ATIVO){
					$row['IDPRODATIVPR'] = self::PRODUTO_INATIVO;
					$row['DSINFORANTERIOR'] = self::PRODUTO_INATIVO;
					$row['DSINFORATUAL'] = self::PRODUTO_ATIVO;
				}else if($row['IDPRODATIVPR'] === self::PRODUTO_INATIVO){
					$row['IDPRODATIVPR'] = self::PRODUTO_ATIVO;
					$row['DSINFORANTERIOR'] = self::PRODUTO_ATIVO;
					$row['DSINFORATUAL'] = self::PRODUTO_INATIVO;
				}
			}else{
				$row['IDPRODATIVPR'] = self::PRODUTO_INATIVO;
				$row['DSINFORANTERIOR'] = self::PRODUTO_INATIVO;
				$row['DSINFORATUAL'] = self::PRODUTO_ATIVO;
			}
			$row['IDANALIZEATIV'] = 'P';
			$this->newWorkflow($request);
			$this->setOpersToWorkflow($request);
			$this->setDadosWflow($request);
			parent::save($request, $response);
			$connection->commit();
		} catch (\Exception $e){
        	$connection->rollback();
            $response->setError(new Error("Error in WORKFLOW process. ". $e->getMessage(), 500));
        }

	}

	private function newWorkflow($request){

		$row = $request->getRow();

		$row['IDTIPOPERWFLOW'] = $row['__is_new'] ? "2" : "1";
		$row['NRPROCESSOAPROVA'] = "2";

		if($row['__is_new'] && $row['changed']){
			$row['NRITEMPROCAPROVA'] = "1";
		}else if(!$row['__is_new'] && $row['changed'] && $row['IDPRODATIVPR'] === self::PRODUTO_INATIVO){
			$row['NRITEMPROCAPROVA'] = "1";
		}else{
			$row['NRITEMPROCAPROVA'] = "2";
		}
  		$row['__is_new'] = true;
		$row['DSMOTIVOWFLOW'] = $row['DSMOTIVOACAO'];
  		$row['NMITEMWFLOW'] = $row['NMPRODUTO'];
  		$row['CDOPERADOR'] = $this->environment->getCdOperador();
  		$row['NRWORKFLOW'] = $this->newCode->getCodeByTableName('WORKFLOW', $this->environment->getNrOrgTrab(), 1, 1, true);
  		$rows = $this->crudService->getRowsToSave($request, "workflow");
		$dataSet = new DataSet('workflow', $rows);
		$this->dataSourceManager->persist($dataSet);

	}

	private function setOpersToWorkflow($request){

		$row = $request->getRow();

		$filter = array(
			array(
				"columnName" => "NRPROCESSOAPROVA",
				"operator" => "=",
				"value" => $row['NRPROCESSOAPROVA']
			),
			array(
				"columnName" => "NRITEMPROCAPROVA",
				"operator" => "=",
				"value" => $row['NRITEMPROCAPROVA']
			)
		);
		
		$filterCriteria = new FilterCriteria('operitempaprova', $filter);
		$filterCriteria =  $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
		$dataSet = $this->dataSourceManager->findBy($filterCriteria);
		$registros = $dataSet->getRows();
		if(empty($registros))
			throw new \Exception("Nenhum operador cadastrado!", 1);
		else {
			$opersWorkflowToSave = array();
			foreach ($registros as $rowOper) {
				$rowOper['IDSITOPERWFLOW'] = "P";
				$rowOper['DTLIMITEWFLOW'] = date('d/m/Y', mktime(0, 0, 0, date('m'), date('d')+$rowOper['NRDIASOPERAPRO'], date('Y')));
				$rowOper['NRWORKFLOW'] = $row['NRWORKFLOW'];
				$rowOper['__is_new'] = true;
				$rowOper['changed'] = true;
				array_push($opersWorkflowToSave, $rowOper);
			}
			$newDatasetToSave = new DataSet('operworkflow', $opersWorkflowToSave);
			$newRequestToSave = new DTO\Request\DataSet($newDatasetToSave, '', '', '');
			$opersWorkflowToSave = $this->crudService->getRowsToSave($newRequestToSave, "operworkflow");
			$dataSetToSave = new DataSet('operworkflow', $opersWorkflowToSave);
			$this->dataSourceManager->persist($dataSetToSave);
		}
	}

	private function setDadosWflow($request){

		$row = $request->getRow();

		$row['NMTABELAWFLOW'] = "PRODUTO";
		$row['NMCAMPOWFLOW'] = "IDPRODATIVPR";
		$row['DSPRIMARYKEY'] = "CDPRODUTO=".$row['CDPRODUTO'].";NRORG=".$this->environment->getNrOrgTrab();

		$dadosWflowToSave = $this->crudService->getRowsToSave($request, "dadoswflow");
		$dataSetToSave = new DataSet('dadoswflow', $dadosWflowToSave);
		$this->dataSourceManager->persist($dataSetToSave);

	}

}