<?php
/**
 * Created by PhpStorm.
 * User: matheussilva
 * Date: 31/07/14
 * Time: 14:20
 */

namespace Folha\Logic\ValesBeneficios\CalculoValeTransporte;

use \Folha\Entities\FpaItecalcfolha;

use Folha\Entities\FpaTipoassoceve;
use Folha\Entities\FpaTipomoviment;
use \Folha\Entities\GpeVinculom;

use \Folha\Entities\GpeVinculoh;

use \Folha\Entities\VabHistvalevinc;

use Folha\Entities\VabIntegtransph;

use \Folha\Entities\VabLinhavinc;

use Folha\Logic\FPATeste;
use \Geral\Util\Ambiente;

use \HCM\Util\DateUtil;

use \Geral\Util\NovoCodigo;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;


class Calculo {

    /**Definição de variáveis de instância
     * @var GpeVinculom $vinculos
     */
    private $nrorg;
    private $vinculos;
    private $dataPagamento;
    private $dataInicio;
    private $dataFinal;
    private $competencia;
    private $nrtipolinhavinc;
    private $hcmParameters;

    /**
     * @param String $vincInicial
     * @param String $vincFinal
     * @param String $dataInicio
     * @param String $dataFinal
     * @param String $dataPagamento
     * @param int $nrorg
     * @param String $dtmescompetencia
     * @param String $ordem
     * @param int $nrtipolinhavinc
     * @throws \Exception
     */
    function __construct(&$response, $idMemoria, $vincInicial, $vincFinal, $dataInicio, $dataFinal, $dataPagamento, $nrorg, $dtmescompetencia, $ordem, $nrtipolinhavinc, $hcmParameters, $row)
    {
        $this->response = $response;
        $this->idMemoria = $idMemoria;
        $this->dataInicio = DateUtil::getDataDeString(substr($dataInicio, 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
        $this->dataFinal = DateUtil::getDataDeString(substr($dataFinal, 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
        $this->dataPagamento = DateUtil::getDataDeString(substr($dataPagamento, 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
        $this->nrorg = $nrorg;
        $this->competencia = $dtmescompetencia;
        $this->nrtipolinhavinc = $nrtipolinhavinc;
        $this->hcmParameters = $hcmParameters;
        $this->row = $row;
        $this->vinculos = $this->buscaVinculo($vincInicial, $vincFinal, $dtmescompetencia, $ordem);
    }

    /**
     * Busca os vínculos que serão calculados
     * @param String $inicial
     * @param String $final
     * @param String $competencia
     * @param String $ordem
     * @return mixed
     * @throws \Exception
     */
    private function buscaVinculo($inicial, $final, $competencia, $ordem)
    {
        try {

            if($competencia instanceof \DateTime){
                $dtmescompetencia = $competencia;
            }else{
                $dtmescompetencia = DateUtil::getDataDeString($competencia, DateUtil::FORMATO_BRASILEIRO, true);
            }
            
            //18/09/20 alteração para olhar vínculos ativos na competência do operador e não na competência + 1 mês ($dtiniperiodo) como era feito
            $inicompetencia = $dtmescompetencia->format('d/m/Y');
            $fimcompetencia = $dtmescompetencia->format('t/m/Y');

            $dtmescompetencia->add(new \DateInterval('P1M'));
            $dtiniperiodo = $dtmescompetencia->format('d/m/Y');
            $dtfinalvale = $this->dataFinal->format('d/m/Y');

            
            // Filtra somente vínculos que possuam histórico
            $vinculosComHistorico = " AND EXISTS (SELECT 1 FROM GPE_VINCULOH WHERE NRVINCULOM = e.NRVINCULOM AND NRORG = e.NRORG and DTMESCOMPETENC <> '01/01/3000') ";
            
            // Filtra somente vínculos que não estejam em situação funcional de afastamento (exceto atestado médico)
            $cond_afastados =
                " AND NOT EXISTS (SELECT * FROM GPE_ALTESITUFUNC
                                   WHERE NRORG         = e.NRORG
                                     AND NRVINCULOM    = e.NRVINCULOM
                                     AND NRSITUFUNCM IN (SELECT GPE_SITUFUNCH.NRSITUFUNCM
                                                           FROM GPE_SITUFUNCH
                                                          WHERE GPE_SITUFUNCH.IDGERVALETRANSP = 'N'
                                                            AND GPE_SITUFUNCH.NRSITUFUNCM NOT IN (12, 15, 16, 13)
                                                            AND GPE_SITUFUNCH.DTEFETIVA = ( SELECT MAX(DTEFETIVA) DTEFETIVA
                                                                                              FROM GPE_SITUFUNCH MAX_SITUFUNCH
                                                                                             WHERE MAX_SITUFUNCH.DTEFETIVA <= TO_DATE('$dtfinalvale','DD/MM/YYYY')
                                                                                               AND MAX_SITUFUNCH.NRSITUFUNCM = GPE_SITUFUNCH.NRSITUFUNCM))
                                     AND DTINISITUFUNC = (SELECT MAX(DTINISITUFUNC)
                                                            FROM GPE_ALTESITUFUNC B
                                                           WHERE B.NRORG       = e.NRORG
                                                             AND B.NRVINCULOM  = e.NRVINCULOM
                                                             AND DTINISITUFUNC <= TO_DATE('$dtfinalvale','DD/MM/YYYY'))
                                     AND (DTFIMSITUFUNC >= TO_DATE('$dtfinalvale','DD/MM/YYYY') OR DTFIMSITUFUNC IS NULL))";
            switch($ordem)
            {
                case 'ALFABETICA':
                {
                    $param = array(':P_INICIO' => $inicial,
                        ':P_FIM' => $final,
                        ':P_NRORG' => $this->nrorg);

                    //Nome Inicial
                    $whereinivinculo = "NRORG = :P_NRORG AND NRVINCULOM = :P_INICIO"; /** @var GpeVinculom $objinivinculo */
                    $objinivinculo = \Folha\Entities\GpeVinculom::getObjectByWhere($whereinivinculo, $param);

                    //Nome Final
                    $wherefimvinculo = "NRORG = :P_NRORG AND NRVINCULOM = :P_FIM"; /** @var GpeVinculom $objfimvinculo */
                    $objfimvinculo = \Folha\Entities\GpeVinculom::getObjectByWhere($wherefimvinculo, $param);

                    $nomeInicio = $objinivinculo->getNmvinculom();
                    $nomeFinal = $objfimvinculo->getNmvinculom();

                    if($nomeFinal<$nomeInicio){
                        $aux = $nomeFinal;
                        $nomeFinal = $nomeInicio;
                        $nomeInicio = $aux;
                    }

                    $param = array( ':P_INICIO' => $nomeInicio,
                                    ':P_FIM' => $nomeFinal,
                                    ':P_NRORG' => $this->nrorg);
                    //$where = "(DTRESCISAOVINC > '$dtiniperiodo' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$dtiniperiodo' AND NMVINCULOM BETWEEN :P_INICIO AND :P_FIM AND NRORG = :P_NRORG";
                    $where = "(DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$fimcompetencia' AND NMVINCULOM BETWEEN :P_INICIO AND :P_FIM AND NRORG = :P_NRORG";
                    $msgerro = 'Não foram encontrados vínculos para os nomes informados';
                    break;
                }
                case 'MATRICULA':{
                    $param = array(
                        'P_INICIO' => $inicial,
                        'P_FIM' => $final,
                        'P_NRORG' => $this->nrorg);
					//$where = "(DTRESCISAOVINC > '$dtiniperiodo' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$dtiniperiodo' AND CDMATRICULA BETWEEN :P_INICIO AND :P_FIM AND NRORG = :P_NRORG";
					$where = "(DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$fimcompetencia' AND CDMATRICULA BETWEEN :P_INICIO AND :P_FIM AND NRORG = :P_NRORG";
                    $msgerro = 'Não foram encontrados vínculos para as matrículas informadas';
                    break;
                }
                case 'DATA_ADMISSAO':{
                    $param = array(':P_NRORG' => $this->nrorg);
                    //$where = "(DTRESCISAOVINC > '$dtiniperiodo' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC BETWEEN '$inicial' AND '$final' AND NRORG = :P_NRORG";
                    $where = "(DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC BETWEEN '$inicial' AND '$final' AND NRORG = :P_NRORG";
                    $msgerro = 'Não foram encontrados vínculos para as datas informadas';
                    break;
                }
                case 'ESTRUTURA':{
                    $param = array( 'P_NRORG' => $this->nrorg,
                                    'P_NRESTRUTLEGAL' =>$inicial,
                                    'P_NRESTRUTGEREN' =>$final);
                    //$where = "NRORG = :P_NRORG AND (DTRESCISAOVINC > '$dtiniperiodo' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$dtiniperiodo' AND NRVINCULOM IN ( SELECT NRVINCULOM FROM GPE_VINCULOH WHERE NRORG = :P_NRORG AND (NRESTRUTGEREN = :P_NRESTRUTGEREN OR :P_NRESTRUTGEREN IS NULL) AND NRESTRUTLEGAL = :P_NRESTRUTLEGAL AND DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC)DTMESCOMPETENC FROM GPE_VINCULOH MAX_VINCULOH WHERE MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM AND MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG))";
                    $where = "NRORG = :P_NRORG AND (DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$fimcompetencia' AND NRVINCULOM IN ( SELECT NRVINCULOM FROM GPE_VINCULOH WHERE NRORG = :P_NRORG AND (NRESTRUTGEREN = :P_NRESTRUTGEREN OR :P_NRESTRUTGEREN IS NULL) AND NRESTRUTLEGAL = :P_NRESTRUTLEGAL AND DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC)DTMESCOMPETENC FROM GPE_VINCULOH MAX_VINCULOH WHERE MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM AND MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG))";
                    $msgerro = 'Não foram encontrados vínculos para as estruturas informadas';
                    break;
                }
                case 'LISTA_VINCULO':
                    $param = array('P_NRORG' => $this->nrorg);
                    //$where = "NRORG = :P_NRORG AND (DTRESCISAOVINC > '$dtiniperiodo' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$dtiniperiodo' AND NRVINCULOM IN ($inicial)";
                    $listaVinculo = isset($this->row['NRVINCULOM_LIST']) && is_array($this->row['NRVINCULOM_LIST']) && count($this->row['NRVINCULOM_LIST']) ? implode(",", $this->row['NRVINCULOM_LIST']) : null;
                    $where = "NRORG = :P_NRORG AND (DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$fimcompetencia' AND NRVINCULOM IN ($listaVinculo)";
                    $msgerro = 'Não foram encontrados vínculos para a lista informada';
                    break;
                case 'LISTA_ESTRUTURA':
                    $listaEstrutLegal = isset($this->row['LIST_NRESTRUTLEGAL']) && is_array($this->row['LIST_NRESTRUTLEGAL']) && count($this->row['LIST_NRESTRUTLEGAL']) ? implode(",", $this->row['LIST_NRESTRUTLEGAL']) : null;
                    $listaEstrutGeren = isset($this->row['LIST_NRESTRUTGEREN']) && is_array($this->row['LIST_NRESTRUTGEREN']) && count($this->row['LIST_NRESTRUTGEREN']) ? implode(",", $this->row['LIST_NRESTRUTGEREN']) : 1;

                    $param = array( 'P_NRORG' => $this->nrorg);
                                    
                    //$where = "NRORG = :P_NRORG AND (DTRESCISAOVINC > '$dtiniperiodo' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$dtiniperiodo' AND NRVINCULOM IN ( SELECT NRVINCULOM FROM GPE_VINCULOH WHERE NRORG = :P_NRORG AND ( NRESTRUTGEREN IN($listaEstrutGeren) OR  1 IN($listaEstrutGeren)  ) AND NRESTRUTLEGAL IN($listaEstrutLegal) AND DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC)DTMESCOMPETENC FROM GPE_VINCULOH MAX_VINCULOH WHERE MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM AND MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG))";
                    $where = "NRORG = :P_NRORG AND (DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$fimcompetencia' AND NRVINCULOM IN ( SELECT NRVINCULOM FROM GPE_VINCULOH WHERE NRORG = :P_NRORG AND ( NRESTRUTGEREN IN($listaEstrutGeren) OR  1 IN($listaEstrutGeren)  ) AND NRESTRUTLEGAL IN($listaEstrutLegal) AND DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC)DTMESCOMPETENC FROM GPE_VINCULOH MAX_VINCULOH WHERE MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM AND MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG))";
                    $msgerro = 'Não foram encontrados vínculos para as estruturas informadas';
                    break;
                case 'LISTA_NRESTRUTSIND':
                    $estrutLegal = isset($this->row['DSINICIOVALE']) && !empty($this->row['DSINICIOVALE']) ? $this->row['DSINICIOVALE'] : null;
                    $listaEstrutSind = isset($this->row['LIST_NRESTRUTSIND']) && is_array($this->row['LIST_NRESTRUTSIND']) && count($this->row['LIST_NRESTRUTSIND']) ? implode(",", $this->row['LIST_NRESTRUTSIND']) : 1;

                    $param = array( 'P_NRORG' => $this->nrorg);
                                    
                    //$where = "NRORG = :P_NRORG AND (DTRESCISAOVINC > '$dtiniperiodo' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$dtiniperiodo' AND NRVINCULOM IN ( SELECT NRVINCULOM FROM GPE_VINCULOH WHERE NRORG = :P_NRORG AND ( NRESTRUTSIND IN($listaEstrutSind) ) AND NRESTRUTLEGAL = $estrutLegal AND DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC)DTMESCOMPETENC FROM GPE_VINCULOH MAX_VINCULOH WHERE MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM AND MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG))";
                    $where = "NRORG = :P_NRORG AND (DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$fimcompetencia' AND NRVINCULOM IN ( SELECT NRVINCULOM FROM GPE_VINCULOH WHERE NRORG = :P_NRORG AND ( NRESTRUTSIND IN($listaEstrutSind) ) AND NRESTRUTLEGAL = $estrutLegal AND DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC)DTMESCOMPETENC FROM GPE_VINCULOH MAX_VINCULOH WHERE MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM AND MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG))";
                    $msgerro = 'Não foram encontrados vínculos para as estruturas informadas';
                    break;
                    
                default: {
                    return null;
                }
            }

            $where .= $vinculosComHistorico . $cond_afastados;
            $historicos = VabHistvalevinc::getObjectsByWhere(
                "NRORG = $this->nrorg AND
                DTMESCOMPAPVALE = '$competencia' AND
                (NRTIPOLINHAVINC = '$this->nrtipolinhavinc' OR NRTIPOLINHAVINC IS NULL) AND
                NRVINCULOM IN (SELECT NRVINCULOM FROM GPE_VINCULOM WHERE $where)", $param
            );
            if ($historicos) {
                throw new \Exception('Vínculo(s) já calculado(s). Favor retroceder.', Messages::ERROR_MESSAGE_CODE);
            }
            $result = \Folha\Entities\GpeVinculom::getObjectsByWhere($where, $param);

            if ($result) {
                return $result;
            } else {
                throw new \Exception($msgerro, Messages::ERROR_MESSAGE_CODE);
            }

        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Inicia o Calculo
     *
     * @throws \Exception
     */
    public function calculo()
    {
        try {
            /**
             * @var \Folha\Entities\GpeVinculom $vinculo
             */
            foreach ($this->vinculos as $vinculo) {
                set_time_limit(0);
                
                MemoriaVT::getInstance()->inicializarDados($vinculo, $this->hcmParameters, $this->dataInicio, $this->dataFinal, $this->dataPagamento);
                
                $params = new ValeVinculo($vinculo, $this->nrorg, $this->dataInicio, $this->dataFinal, $this->nrtipolinhavinc, $this->hcmParameters);
                $infos = $params->getValeTransporte();
                if (!empty($infos)) {
                    foreach ($infos as $info) {
                        $params->addHistorico($info, $vinculo->getNrvinculom(), $this->dataPagamento);
                    }
                }
                MemoriaVT::getInstance()->concatenarMensagemVinc();
            }
            
            MemoriaVT::getInstance()->salvarMsgArquivo();

            if($this->idMemoria == 'S') $this->response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('memoriaCalculo', array(MemoriaVT::getInstance()->getCaminhoArquivo())) );

            \Setforms2\Shared\Registry::getEntityManager()->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Insere os Vinculos no Itecalcfolha
     */
    private function insereItemCalculo($info, $vinculo)
    {

        /**
         * @var FpaItecalcfolha $item
         */
        $item = new FpaItecalcfolha();

        $item->setNritemcalcfolha(NovoCodigo::geraCodigo($this->nrorg, 'FPA_ITECALCFOLHA', 1, 22, true));
        $item->setCdoperinclusao(Ambiente::getCdOperador());
        $item->setCdoperultatu(Ambiente::getCdOperador());
        $item->setDtinclusao(DateUtil::getDataAtual());
        $item->setDtocorrencia(DateUtil::getDataDeString($this->competencia, DateUtil::FORMATO_BRASILEIRO, true));
        $item->setDtultatu(DateUtil::getDataAtual());
        $item->setIdevendemoncalc('N');
        $item->setIdrefevalorcalc('NORMAL');
        $item->setNrcalculofolha(tk_get_env_var('NRCALCULO_VAB'));
        $item->setNreventom(480);
        $item->setNrorg(Ambiente::getNrorgTrabalho());
        $item->setNrorginclusao(Ambiente::getNrorgTrabalho());
        $item->setNrorgultatu(Ambiente::getNrorgTrabalho());
        $item->setNrvinculom($vinculo);
        $item->setQtocorproxperi(1);
        $item->setVreventofolha((float)$info['VALOR']);
        $item->setVrreffolha((int)$info['REFERENCIA']);

        $item->persist();
        \Setforms2\Shared\Registry::getEntityManager()->flush();

    }

    /**
     * Retrocede, apagando todos os históricos gerados.
     * @param $movimento
     * @param $competencia
     * @param $nrorg
     * @param $inicial
     * @param $final
     * @param $ordem
     * @param $nrtipolinhavinc
     * @throws \Exception
     */
    public static function retroceder($movimento, $competencia, $nrorg, $inicial, $final, $ordem, $nrtipolinhavinc, $row)
    {
        try {
            $dtmescompetencia = DateUtil::getDataDeString($competencia, DateUtil::FORMATO_BRASILEIRO, true);
            
            //18/09/20 alteração para olhar vínculos ativos na competência do operador e não na competência + 1 mês ($dtiniperiodo) como era feito
            $inicompetencia = $dtmescompetencia->format('d/m/Y');
            $fimcompetencia = $dtmescompetencia->format('t/m/Y');
            
            $dtmescompetencia->add(new \DateInterval('P1M'));
            $dtiniperiodo = $dtmescompetencia->format('d/m/Y');
            $nrtipomoviment = FpaTipomoviment::TP_MOV_VALE_TRANSPORTE;

            $param = array( 'P_INICIO'       => $inicial,
                            'P_FIM'          => $final,
                            'P_NRORG'        => $nrorg,
                            //'P_DTINIPERIODO' => $dtiniperiodo);
                            'P_INICOMPETENCIA' => $inicompetencia,
                            'P_FIMCOMPETENCIA' => $fimcompetencia);

            switch($ordem)
            {
                case 'ALFABETICA':
                {
                    //$result = \Folha\Entities\GpeVinculom::getObjectsByWhere("(TO_CHAR(DTRESCISAOVINC, 'DD/MM/YYYY') > :P_DTINIPERIODO OR DTRESCISAOVINC IS NULL) AND TO_CHAR(DTADMISSAOVINC, 'DD/MM/YYYY') <= :P_DTINIPERIODO AND NRVINCULOM BETWEEN :P_INICIO AND :P_FIM", $param);
                    //$where = "(DTRESCISAOVINC > :P_DTINIPERIODO OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= :P_DTINIPERIODO AND NRVINCULOM BETWEEN :P_INICIO AND :P_FIM AND NRORG = :P_NRORG";
                    $where = "(DTRESCISAOVINC > :P_INICOMPETENCIA OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= :P_FIMCOMPETENCIA AND NRVINCULOM BETWEEN :P_INICIO AND :P_FIM AND NRORG = :P_NRORG";
                    $result = \Folha\Entities\GpeVinculom::getObjectsByWhere($where, $param);
                    break;
                }
                case 'MATRICULA':{
                    //$where = "(DTRESCISAOVINC > :P_DTINIPERIODO OR DTRESCISAOVINC IS NULL) AND TO_CHAR(DTADMISSAOVINC, 'DD/MM/YYYY') <= :P_DTINIPERIODO AND CDMATRICULA BETWEEN :P_INICIO AND :P_FIM AND NRORG = :P_NRORG";
                    $where = "(DTRESCISAOVINC > :P_INICOMPETENCIA OR DTRESCISAOVINC IS NULL) AND TO_CHAR(DTADMISSAOVINC, 'DD/MM/YYYY') <= :P_FIMCOMPETENCIA AND CDMATRICULA BETWEEN :P_INICIO AND :P_FIM AND NRORG = :P_NRORG";
                    $result = \Folha\Entities\GpeVinculom::getObjectsByWhere($where, $param);
                    break;
                }
                case 'DATA_ADMISSAO':{
                    //$where = "(DTRESCISAOVINC > :P_DTINIPERIODO OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC BETWEEN :P_INICIO AND :P_FIM AND NRORG = :P_NRORG";
                    $where = "(DTRESCISAOVINC > :P_INICOMPETENCIA OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC BETWEEN :P_INICIO AND :P_FIM AND NRORG = :P_NRORG";
                    $result = \Folha\Entities\GpeVinculom::getObjectsByWhere($where, $param);
                    break;
                }
                case 'ESTRUTURA':{
                    //$where = "NRORG = :P_NRORG AND (DTRESCISAOVINC > :P_DTINIPERIODO OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= :P_DTINIPERIODO AND NRVINCULOM IN ( SELECT NRVINCULOM FROM GPE_VINCULOH WHERE NRORG = :P_NRORG AND ((NRESTRUTGEREN = :P_FIM) OR (:P_FIM IS NULL)) AND NRESTRUTLEGAL = :P_INICIO AND DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC)DTMESCOMPETENC FROM GPE_VINCULOH MAX_VINCULOH WHERE MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM AND MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG AND DTMESCOMPETENC <= :P_DTINIPERIODO))";
                    $where = "NRORG = :P_NRORG AND (DTRESCISAOVINC > :P_INICOMPETENCIA OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= :P_FIMCOMPETENCIA AND NRVINCULOM IN ( SELECT NRVINCULOM FROM GPE_VINCULOH WHERE NRORG = :P_NRORG AND ((NRESTRUTGEREN = :P_FIM) OR (:P_FIM IS NULL)) AND NRESTRUTLEGAL = :P_INICIO AND DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC)DTMESCOMPETENC FROM GPE_VINCULOH MAX_VINCULOH WHERE MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM AND MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG AND DTMESCOMPETENC <= :P_FIMCOMPETENCIA))";
                    $result = \Folha\Entities\GpeVinculom::getObjectsByWhere($where, $param);
                    break;
                }
                case 'LISTA_VINCULO':
                    $param = array(
                        'P_NRORG' => $nrorg,
                        'P_INICOMPETENCIA' => $inicompetencia,
                        'P_FIMCOMPETENCIA' => $fimcompetencia,
                    );
                    //$where = "NRORG = :P_NRORG AND (DTRESCISAOVINC > '$dtiniperiodo' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$dtiniperiodo' AND NRVINCULOM IN ($inicial)";
                    $where = "NRORG = :P_NRORG AND (DTRESCISAOVINC > :P_INICOMPETENCIA OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= :P_FIMCOMPETENCIA AND NRVINCULOM IN ($inicial)";
                    $result = \Folha\Entities\GpeVinculom::getObjectsByWhere($where, $param);
                    break;
                case 'LISTA_ESTRUTURA':
                    $listaEstrutLegal = isset($row['LIST_NRESTRUTLEGAL']) && is_array($row['LIST_NRESTRUTLEGAL']) && count($row['LIST_NRESTRUTLEGAL']) ? implode(",", $row['LIST_NRESTRUTLEGAL']) : null;
                    $listaEstrutGeren = isset($row['LIST_NRESTRUTGEREN']) && is_array($row['LIST_NRESTRUTGEREN']) && count($row['LIST_NRESTRUTGEREN']) ? implode(",", $row['LIST_NRESTRUTGEREN']) : 1;

                    $param = array(
                        'P_NRORG' => $nrorg,
                        'P_INICOMPETENCIA' => $inicompetencia,
                        'P_FIMCOMPETENCIA' => $fimcompetencia,
                    );
                                    
                    //$where = "NRORG = :P_NRORG AND (DTRESCISAOVINC > '$dtiniperiodo' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$dtiniperiodo' AND NRVINCULOM IN ( SELECT NRVINCULOM FROM GPE_VINCULOH WHERE NRORG = :P_NRORG AND ( NRESTRUTGEREN IN($listaEstrutGeren) OR  1 IN($listaEstrutGeren)  ) AND NRESTRUTLEGAL IN($listaEstrutLegal) AND DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC)DTMESCOMPETENC FROM GPE_VINCULOH MAX_VINCULOH WHERE MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM AND MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG))";
                    $where = "NRORG = :P_NRORG AND (DTRESCISAOVINC > :P_INICOMPETENCIA OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= :P_FIMCOMPETENCIA AND NRVINCULOM IN ( SELECT NRVINCULOM FROM GPE_VINCULOH WHERE NRORG = :P_NRORG AND ( NRESTRUTGEREN IN($listaEstrutGeren) OR  1 IN($listaEstrutGeren)  ) AND NRESTRUTLEGAL IN($listaEstrutLegal) AND DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC)DTMESCOMPETENC FROM GPE_VINCULOH MAX_VINCULOH WHERE MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM AND MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG))";
                    $result = \Folha\Entities\GpeVinculom::getObjectsByWhere($where, $param);
                    break;
                case 'LISTA_NRESTRUTSIND':
                    $estrutLegal = isset($row['DSINICIOVALE']) && !empty($row['DSINICIOVALE']) ? $row['DSINICIOVALE'] : null;
                    $listaEstrutSind = isset($row['LIST_NRESTRUTSIND']) && is_array($row['LIST_NRESTRUTSIND']) && count($row['LIST_NRESTRUTSIND']) ? implode(",", $row['LIST_NRESTRUTSIND']) : 1;

                    $param = array(
                        'P_NRORG' => $nrorg,
                        'P_INICOMPETENCIA' => $inicompetencia,
                        'P_FIMCOMPETENCIA' => $fimcompetencia,
                    );
                                    
                    //$where = "NRORG = :P_NRORG AND (DTRESCISAOVINC > '$dtiniperiodo' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$dtiniperiodo' AND NRVINCULOM IN ( SELECT NRVINCULOM FROM GPE_VINCULOH WHERE NRORG = :P_NRORG AND ( NRESTRUTSIND IN($listaEstrutSind) ) AND NRESTRUTLEGAL = $estrutLegal AND DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC)DTMESCOMPETENC FROM GPE_VINCULOH MAX_VINCULOH WHERE MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM AND MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG))";
                    $where = "NRORG = :P_NRORG AND (DTRESCISAOVINC > :P_INICOMPETENCIA OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= :P_FIMCOMPETENCIA AND NRVINCULOM IN ( SELECT NRVINCULOM FROM GPE_VINCULOH WHERE NRORG = :P_NRORG AND ( NRESTRUTSIND IN($listaEstrutSind) ) AND NRESTRUTLEGAL = $estrutLegal AND DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC)DTMESCOMPETENC FROM GPE_VINCULOH MAX_VINCULOH WHERE MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM AND MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG))";
                    $result = \Folha\Entities\GpeVinculom::getObjectsByWhere($where, $param);
                    break;
            }

            /**
             * @var \Folha\Entities\GpeVinculom $reg
             * @var VabHistvalevinc $hist
             */
            if($result){
                /*$vinculos = array();
                foreach($result as $reg){
                    $vinculos[] = $reg->getNrvinculom();
                }
                $vinculos = implode(', ', $vinculos);*/

                /* Verifica se já houve exportação do vale para a folha */
                /*$exportado = self::verificaExportadosFolha($nrorg, $vinculos);

                if ($exportado) {
                    throw new \Exception("Não foi possível retroceder. Existe(m) vínculo(s) exportado(s) para a folha.");
                }*/

                /*$itens = FpaItecalcfolha::getObjectsByWhere(
                    "NRORG = $nrorg AND
                     NRCALCULOFOLHA  = (SELECT NRCALCULOFOLHA FROM FPA_CALCULOFOLHA WHERE NRCALCULOFOLHA = $movimento AND NRORG = $nrorg AND NRTIPOMOVIMENT = $nrtipomoviment) AND
                     NREVENTOM IN (3030, 3040, 480) AND
                     NRVINCULOM IN ($vinculos)", array());*/
                $itens = array();

                $hists = VabHistvalevinc::getObjectsByWhere(
                    "NRORG = $nrorg AND
                     TRUNC(DTMESCOMPAPVALE, 'DD') = '$competencia' AND
                     NRCALCULOFOLHA  = (SELECT NRCALCULOFOLHA FROM FPA_CALCULOFOLHA WHERE NRCALCULOFOLHA = $movimento AND NRORG = $nrorg AND NRTIPOMOVIMENT = $nrtipomoviment) AND
                     NRVINCULOM IN (SELECT NRVINCULOM FROM GPE_VINCULOM WHERE $where) AND
                     (NRTIPOLINHAVINC = '$nrtipolinhavinc' OR '$nrtipolinhavinc' IS NULL)", $param);

                if($itens || $hists){
                    foreach ($itens as $itemcalc) {
                        $itemcalc->delete();
                    }
                    foreach($hists as $hist){
                        $hist->delete();
                    }
                    \Setforms2\Shared\Registry::getEntityManager()->flush();
                } else {
                    throw new \Exception('Não foram encontrados lançamentos para retroceder.', Messages::ERROR_MESSAGE_CODE);
                }

            } else {
                throw new \Exception('Não foram encontrados vínculos para o critério de seleção.', Messages::ERROR_MESSAGE_CODE);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private static function verificaExportadosFolha($nrorg, $vinculos)
    {
        $nrcalculomensal = HCMParameters::determinaNrCalculoFolha(FpaTipomoviment::TP_MOV_MENSAL);

        $params = array(
            'P_NREVENTOM'       => FpaTipoassoceve::ASSOC_VALE_TRANSP_RECEB,
            'P_NRCALCULOFOLHA'  => $nrcalculomensal,
            'P_NRORG'           => $nrorg
        );
        $where = "NREVENTOM = :P_NREVENTOM AND NRCALCULOFOLHA = :P_NRCALCULOFOLHA AND NRORG = :P_NRORG AND NRVINCULOM IN ($vinculos)";
        $itemcalcs = FpaItecalcfolha::getObjectsByWhere($where, $params);

        if($itemcalcs){
            return true;
        }else{
            return false;
        }
    }
} 